/*
    t5.c
*/

#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
/* #include <windows.h> */

char* iname;     /* input filename */
char oname[64];  /* output filename */
int f_v = 0;     /* verbose flag */

/*
    HwIl formatted output (@\)
*/
char* eng_fmt(char* s0, double a, int prec, int f)
{
    const char e[] = "afpnum kMGTPE";
    char* s = s0;
    if (a < 0) {
        a = -a;
        *s++ = '-';
    }
    if (a < 1e-18 || a >= 1e21) {
        sprintf(s0, "%g", a);
        return s0;
    }
    double dx = log10(1e18 * a);
    int ix = (int)dx / 3 * 3 - 18;
    int ip = prec - ((int)dx % 3) - 1;
    if (ix) {
        if (f)
            sprintf(s, "%.*f%c", ip, a/pow(10, ix), e[(ix+18)/3]);
        else
            sprintf(s, "%.*fe%d", ip, a/pow(10, ix), ix);
    } else
        sprintf(s, "%.*f", ip, a);
    return s0;
}

FILE *ifp[2], *ofp;
char buf[4096];

void tolower_line(char* s)
{
    while (*s) {
        *s = tolower(*s);
        s += 1;
    }
}

struct vect {
    int n;
    double f, c, nc, p;
} d[2];

const char sfmt[] = "%d %lf %lf %lf %lf";

int p3()
{
    int f0, f1;
    char cb[2][32];
    do {
        double ang = d[0].p - d[1].p - 180;
        if (ang < -360)
            ang += 360;
        sprintf(buf, "%sHz  %sdB  %g deg.\n",
          eng_fmt(cb[0], d[0].f, 4, 1),
          eng_fmt(cb[1], 20*log10(d[0].c/d[1].c), 4, 1), ang);
        fputs(buf, ofp);
        printf(buf);
        if (f_v) {
            printf(" x: %d %g %g %g\n", d[0].n, d[0].f, d[0].c, d[0].p);
            printf(" y: %d %g %g %g\n", d[1].n, d[1].f, d[1].c, d[1].p);
        }
        f0 = f1 = 0;
        if (fgets(buf, sizeof(buf), ifp[0]))
            if (sscanf(buf, sfmt, &d[0].n, &d[0].f, &d[0].c, &d[0].nc, &d[0].p) == 5)
                f0 = 1;
        if (fgets(buf, sizeof(buf), ifp[1]))
            if (sscanf(buf, sfmt, &d[1].n, &d[1].f, &d[1].c, &d[1].nc, &d[1].p) == 5)
                f1 = 1;
    } while (f0 && f1);
    return 0;
}

int p2()
{
    char* s;
    fseek(ifp[1], ftell(ifp[0]), SEEK_SET);
    while ((s = fgets(buf, sizeof(buf), ifp[1]))) {
        tolower_line(buf);
        if (strstr(buf, "v(y)")) {
            while ((s = fgets(buf, sizeof(buf), ifp[0]))) {
                if (sscanf(buf, sfmt,
                  &d[0].n, &d[0].f, &d[0].c, &d[0].nc, &d[0].p) == 5) {
                    /* printf("x: %s", buf); */
                    while ((s = fgets(buf, sizeof(buf), ifp[1]))) {
                        if (sscanf(buf, sfmt,
                          &d[1].n, &d[1].f, &d[1].c, &d[1].nc, &d[1].p) == 5) {
                            /* printf("y: %s", buf); */
                            return p3();
                        }
                    }
                    printf("Corresponding V(y) vector not found.\n");
                    return -1;
                }
            }
            printf("Unexpected EOF.\n");
            return -1;
        }
    }
    printf("V(y) not found.\n");
    return -1;
}

int p1()
{
    fprintf(ofp, "%s -> %s\n", iname, oname);
    int fnd = 0;
    char* s;
    while ((s = fgets(buf, sizeof(buf), ifp[0]))) {
        tolower_line(buf);
        if (strstr(buf, "v(x)")) {
            fnd = 1;
            if (p2())
                return -1;
        }
    }
    if (!fnd) {
        printf("V(x) not found.\n");
        return -1;
    }
    return 0;
}

int p0()
{
    const char fc[] = "File %s not found.\n";
    if ((ifp[0] = fopen(iname, "r")) == NULL) {
        printf(fc, iname);
        return -1;
    }
    if ((ifp[1] = fopen(iname, "r")) == NULL) {
        printf(fc, iname);
        return -1;
    }
    if ((ofp = fopen(oname, "w")) == NULL) {
        printf("Cannot open file %s for output.\n", oname);
        return -1;
    }
    int r = p1();
    if (fclose(ofp)) {
        printf("fclose %s fail.\n", oname);
        r |= 1;
    }
    fclose(ifp[1]);
    fclose(ifp[0]);
    return (r) ? -1 : 0;
}

/*
    R}hCIvV̎擾
*/
void getopts(char *argv[]) {
    while (*++argv) {
        if (**argv == '-') {
            char *opts[] = { "v" };
            int i, found = 0;
            for (i = 0; i < sizeof(opts)/sizeof(opts[0]); i += 1)
                if (strcmp((*argv + 1), opts[i]) == 0) {
                    switch (i) {
                    case 0: f_v = 1; break;
                    }
                    found = 1;
                }
            if (!found)
                printf("Unknown option \"%s\".\n", *argv);
        } else {
            iname = *argv;
            char* s = iname;
            char* d = oname;
            while (*s && (d - oname) < sizeof(oname)-1)
                *d++ = *s++;
            while (*(d-1) != '.' && d > oname)
                d -= 1;
            strcpy(d, "txt");
            if (f_v)
                printf("# %s -> %s\n", iname, oname);
        }
    }
}

int main(int argc, char *argv[])
{
    const char* usg = "Usage: t5 [-v] logfile\n";
    if (argc < 2) {
        printf(usg);
        return 255;
    }
    getopts(argv);
    if (!*iname) {
        printf(usg);
        return 255;
    }
    return p0() ? 255 : 0;
}
